import hashlib
import json
import os
import shutil
from dataclasses import dataclass, asdict
from json.decoder import JSONDecodeError
from pathlib import Path
from typing import List, Union
from urllib.parse import urljoin

import requests
from dacite import from_dict, Config

from vankrupt_workshop_client.api import (
    _env_map,
    APIEnv,
    Env,
)
from vankrupt_workshop_client.api import api_entities


@dataclass
class ChunkInOldFile:
    chunk: api_entities.Chunk
    offset_in_file: int


@dataclass
class UpdatePlan:
    items: List[Union[ChunkInOldFile, api_entities.Chunk]]


@dataclass
class InternalSequencedChunk(api_entities.SequencedChunk):
    offset: int


class ConsumerClientException(Exception):
    message: str

    def __init__(self, message):
        self.message = message

    def __str__(self):
        return self.message


def _calculate_chunk_offsets(chunks: List[api_entities.SequencedChunk]) -> List[InternalSequencedChunk]:
    acc = 0
    res = []
    for i, ch in enumerate(chunks):
        if ch.chunk_sequence != i:
            raise ConsumerClientException(
                message=f'An error in chunk sequences found at calculate_chunk_offsets: {chunks}'
            )

        res.append(InternalSequencedChunk(offset=acc, chunk_sequence=i, chunk=ch.chunk))
        acc += ch.chunk.true_size_bytes
    return res


def _plan_update(new_file: api_entities.ModFileDownload, old_file: api_entities.ModFileDownload) -> UpdatePlan:
    sorted(new_file.chunks, key=lambda x: x.chunk_sequence)
    sorted(old_file.chunks, key=lambda x: x.chunk_sequence)

    old_file_with_offsets = _calculate_chunk_offsets(old_file.chunks)
    old_ids = {x.chunk.id: x for x in old_file_with_offsets}

    plan = []
    for i, chunk in enumerate(new_file.chunks):
        if chunk.chunk_sequence != i:
            raise ConsumerClientException(message=f'An error in chunk sequences found: {new_file}')

        # if found in an old file - use it
        if chunk.chunk.id in old_ids.keys():
            plan.append(ChunkInOldFile(
                chunk=old_ids[chunk.chunk.id].chunk,
                offset_in_file=old_ids[chunk.chunk.id].offset
            ))
        else:
            plan.append(
                chunk.chunk
            )

    return UpdatePlan(items=plan)


def _plan_download(new_file: api_entities.ModFileDownload) -> UpdatePlan:
    sorted(new_file.chunks, key=lambda x: x.chunk_sequence)
    plan = []

    for i, chunk in enumerate(new_file.chunks):
        if chunk.chunk_sequence != i:
            raise ConsumerClientException(message=f'An error in chunk sequences found: {new_file}')
        plan.append(
            chunk.chunk
        )
    return UpdatePlan(items=plan)


class ConsumerClient:
    env: APIEnv
    env_vars: Env

    def __init__(self, env: APIEnv = APIEnv.release):
        self.env = env
        self.env_vars = _env_map[env]

    def _get_chunk_download_url(self, mod_id: int, chunk: api_entities.Chunk):
        url = self.env_vars.chunk_url_template.substitute(mod_id=mod_id, hash=chunk.custom_hash)
        return url

    def _download_and_verify_chunk(self, url: str, chunk: api_entities.Chunk) -> bytes:
        blob = requests.get(url).content
        assert len(blob) == chunk.storage_used_bytes
        raw = self.env_vars.decompress_func(blob)
        assert len(raw) == chunk.true_size_bytes
        assert self.env_vars.hash_func(raw).hexdigest() == chunk.custom_hash
        return raw

    def _cut_and_verify_chunk(self, file: str, chunk: ChunkInOldFile) -> bytes:
        with open(file, 'rb') as f:
            f.seek(chunk.offset_in_file, os.SEEK_SET)
            raw = f.read(chunk.chunk.true_size_bytes)
        assert self.env_vars.hash_func(raw).hexdigest() == chunk.chunk.custom_hash
        return raw

    def _execute_plan(
            self,
            plan: UpdatePlan,
            mods_home_dir: str,
            mod_id: int,
            platform: api_entities.GamingPlatform,
            file_to_save: api_entities.ModFileDownload,
    ):
        root = Path.joinpath(Path(mods_home_dir), str(mod_id), platform.value)
        downloaded = reused = 0

        # save new file to .../bin2
        with open(Path.joinpath(root, 'bin2'), 'wb') as write_to:
            for item in plan.items:
                if isinstance(item, api_entities.Chunk):
                    url = self._get_chunk_download_url(mod_id=mod_id, chunk=item)
                    try:
                        raw = self._download_and_verify_chunk(url, chunk=item)
                        downloaded += 1
                    except AssertionError as e:
                        raise ConsumerClientException(message=f'Chunk assertion fail: {e}')
                elif isinstance(item, ChunkInOldFile):
                    try:
                        old_file_path = Path.joinpath(root, 'bin')
                        raw = self._cut_and_verify_chunk(file=old_file_path, chunk=item)
                        reused += 1
                    except AssertionError as e:
                        raise ConsumerClientException(
                            message=f'Assertion fail when reusing chunk, most probably, save file is corrupted: {e}'
                        )
                write_to.write(raw)
        print(f"reused: {reused}, downloaded: {downloaded}")

        # confirm resulting file's md5 hash just in case
        with open(Path.joinpath(root, 'bin2'), 'rb') as a_file:
            content = a_file.read()
            md5_hash = hashlib.md5()
            md5_hash.update(content)
            digest = md5_hash.hexdigest()
            if not digest == file_to_save.file_md5_hash:
                raise ConsumerClientException(
                    message=f'Resulting file hash mismatch. Got: {digest}, expected: {file_to_save.file_md5_hash}'
                )

        # delete .../bin file
        Path.joinpath(root, 'bin').unlink(missing_ok=True)
        # rename .../bin2 to .../bin
        Path.rename(
            Path.joinpath(root, 'bin2'),
            Path.joinpath(root, 'bin'),
        )
        # save new metadata
        with open(Path.joinpath(root, 'meta.txt'), 'w') as metadata_dump:
            w_dict = asdict(file_to_save)
            w_str = json.dumps(w_dict)
            metadata_dump.write(w_str)

    def filter_mods(
            self,
            platform: api_entities.GamingPlatform,
            page: int,
            size: int,
    ) -> api_entities.Page[api_entities.ModBase]:
        """
        Get mods for a platform.

        :param platform: the platform to filter mods for
        :param page:
        :param size:
        :return:
        """
        resp = requests.get(
            url=urljoin(
                base=self.env_vars.api_url,
                url=f'mods/platform/{platform}/?page={page}&size={size}'
            )
        )
        resp.raise_for_status()
        resp_json = resp.json()
        mods = from_dict(data_class=api_entities.Page, data=resp_json, config=Config(check_types=False))
        return mods

    def inspect_mod(
            self,
            mod_id: int,
    ) -> api_entities.ModInspect:
        """
        Inspect a mod, without all the download metadata.

        :param mod_id:
        :return:
        """
        resp = requests.get(
            url=urljoin(
                base=self.env_vars.api_url,
                url=f'mods/{mod_id}/with_files/'
            )
        )
        resp.raise_for_status()
        resp_json = resp.json()
        for f in resp_json['files']:
            f['platform'] = api_entities.GamingPlatform[f['platform']]
        mod = from_dict(data_class=api_entities.ModInspect, data=resp_json)
        return mod

    def get_mod_download_metadata(
            self,
            platform: api_entities.GamingPlatform,
            mod_id: int,
    ) -> api_entities.ModDownload:
        """
        Get all the metadata needed to download the mod.

        :param platform:
        :param mod_id:
        :return:
        """
        resp = requests.get(
            url=urljoin(
                base=self.env_vars.api_url,
                url=f'mods/{mod_id}/{platform.value}/download/'
            )
        )
        resp.raise_for_status()
        resp_json = resp.json()
        for f in resp_json['files']:
            f['platform'] = api_entities.GamingPlatform[f['platform']]
        mod = from_dict(data_class=api_entities.ModDownload, data=resp_json)
        return mod

    def save_mod(
            self,
            mods_home_dir: str,
            platform: api_entities.GamingPlatform,
            mod_id: int,
    ):
        """
        Download, extract, install and verify the installation of the mod.

        :param mods_home_dir:
        :param platform:
        :param mod_id:
        :return:
        """
        root = Path.joinpath(Path(mods_home_dir), str(mod_id), platform.value)

        if root.is_dir():
            raise IsADirectoryError("File directory not empty, either delete or choose update.")
        else:
            root.mkdir(parents=True)

        mod = self.get_mod_download_metadata(mod_id=mod_id, platform=platform)
        file = mod.files[0]
        plan = _plan_download(new_file=file)
        self._execute_plan(
            plan=plan,
            mods_home_dir=mods_home_dir,
            mod_id=mod_id,
            platform=platform,
            file_to_save=file
        )

    def update_mod(
            self,
            mods_home_dir: str,
            platform: api_entities.GamingPlatform,
            mod_id: int,
    ):
        """
        Download, extract, install and verify the installation of the mod.

        :param mods_home_dir:
        :param platform:
        :param mod_id:
        :return:
        """
        root = Path.joinpath(Path(mods_home_dir), str(mod_id), platform.value)

        if not root.is_dir():
            raise NotADirectoryError("File directory empty, download the mod first.")
        mod = self.get_mod_download_metadata(mod_id=mod_id, platform=platform)
        new_file = mod.files[0]
        with open(Path.joinpath(root, 'meta.txt'), 'r') as metadata:
            old_s = metadata.read()
            try:
                old_d = json.loads(old_s)
            except JSONDecodeError:
                raise ConsumerClientException(message='Corrupt metadata, reinstall the mod')
            old_d['platform'] = api_entities.GamingPlatform[old_d['platform']]

            old_file = from_dict(data_class=api_entities.ModFileDownload, data=old_d)
        plan = _plan_update(new_file=new_file, old_file=old_file)
        self._execute_plan(
            plan=plan,
            mods_home_dir=mods_home_dir,
            mod_id=mod_id,
            platform=platform,
            file_to_save=new_file
        )

    @staticmethod
    def delete_mod(
            mods_home_dir: str,
            platform: api_entities.GamingPlatform,
            mod_id: int,
    ):
        path = Path.joinpath(Path(mods_home_dir), str(mod_id), platform.value)
        shutil.rmtree(path)



import enum
from dataclasses import dataclass
from datetime import datetime
from typing import List, Optional, TypeVar, Generic


class GamingPlatform(str, enum.Enum):
    linux = 'linux'
    windows = 'windows'
    android = 'android'
    ps5 = 'ps5'


@dataclass
class Chunk:
    id: int
    true_size_bytes: int
    storage_used_bytes: int
    custom_hash: str
    created_at: str


@dataclass
class SequencedChunk:
    chunk_sequence: int
    chunk: Chunk


@dataclass
class ModFileUploadRequest:
    platform: str
    update_message: str
    size: int


@dataclass
class ModFileUploadResponse:
    platform: str
    update_message: str
    size: int
    id: int
    user_id: int
    signed_policy: dict
    map_file_id: int
    status: str
    error_message: Optional[str]
    created_at: str


@dataclass
class _ModFile:
    platform: 'GamingPlatform'
    version: int
    true_size_bytes: int
    storage_used_bytes: int
    file_md5_hash: str
    created_at: str


@dataclass
class ModFileList(_ModFile):
    ...


@dataclass
class ModFileDownload(_ModFile):
    chunks: List['SequencedChunk']


@dataclass
class ModCreate:
    name: str
    description: str


@dataclass
class ModBase:
    name: str
    description: str
    creator_id: int
    id: int
    created_at: str


@dataclass
class ModList(ModBase):
    ...


@dataclass
class ModInspect(ModBase):
    files: List['ModFileList']


@dataclass
class ModDownload(ModBase):
    files: List['ModFileDownload']


@dataclass
class User:
    name: str
    email: str
    id: int
    is_active: bool
    email_verified: bool


T = TypeVar('T')


@dataclass
class Page(Generic[T]):
    items: List[T]
    total: int
    page: int
    size: int



import base64
import hashlib
import json
from dataclasses import asdict
from urllib.parse import urljoin

import requests
from dacite import from_dict

from vankrupt_workshop_client.api import (
    APIEnv,
    Env,
    _env_map,
    api_entities,
)

md5_hash = hashlib.md5()


class AuthError(ValueError):
    ...


def auth_check(f):
    def wrapper(*args, **kwargs):
        assert args[0].authenticated
        try:
            ans = f(*args, **kwargs)
        except requests.exceptions.HTTPError as e:
            if e.response.status_code != 401:
                raise e

            try:
                args[0].refresh_token()
            except Exception as e:
                print('Got error refreshing access token')
                print(e)
                raise AuthError()

            ans = f(*args, **kwargs)
        return ans

    return wrapper


def notify_dedup_service(url, upload_id, upload_size, file_md5, bucket_name):
    notification = {
        "message": {
            "attributes": {
                "eventType": "OBJECT_FINALIZE",
                "payloadFormat": "JSON_API_V1",
            },
            "data": base64.encodebytes(json.dumps({
                "bucket": bucket_name,
                "id": "random id",
                "selfLink": "str",
                "name": upload_id,
                "size": upload_size,
                "md5Hash": file_md5,
                "timeCreated": "str",
            }).encode()).decode()
        }
    }
    requests.post(
        url,
        json=notification,
    )


class CreatorClient:
    env: APIEnv
    env_vars: Env

    def __init__(
            self,
            env: APIEnv = APIEnv.release,
    ):
        self.env = env
        self.env_vars = _env_map[env]

        self._auth_header = None
        self._access_token = None
        self._refresh_token = None

        self.authenticated = False

    @staticmethod
    def xml_upload(mod_file, policy_document, file_name):
        files = {
            "file": (file_name, mod_file)
        }

        return requests.post(
            policy_document['url'],
            data=policy_document['fields'],
            files=files,
        )

    def local_upload(self, mod_file, object_name):
        """
        curl -X POST --data-binary @OBJECT_LOCATION \
            -H "Authorization: Bearer OAUTH2_TOKEN" \
            -H "Content-Type: OBJECT_CONTENT_TYPE" \
            ""
        """
        return requests.post(
            (f'http://localhost:8081/upload/storage/v1/b/'
             f'{self.env_vars.upload_bucket_name}/o?uploadType=media&name={object_name}'),
            data=mod_file,
        )

    def register(self, user: dict):
        response = requests.post(
            urljoin(self.env_vars.api_url, '/users/'),
            json=user
        )
        return response

    def authenticate(self, username: str, password: str) -> dict:
        response = requests.post(
            urljoin(self.env_vars.api_url, '/token'),
            data={
                "grant_type": "password",
                "username": username,
                "password": password,
            }
        )
        return self._authenticate(response.json()['access_token'], response.json()['refresh_token'])

    def _authenticate(self, a_t, r_t):
        self._auth_header = {
            'Authorization': 'Bearer ' + a_t
        }
        self._access_token = a_t
        self._refresh_token = r_t
        self.authenticated = True
        return self._auth_header

    def refresh_token(self):
        if not self._refresh_token:
            raise EnvironmentError

        response = requests.post(
            urljoin(self.env_vars.api_url, '/refresh-token'),
            json={
                "refresh_token": self._refresh_token,
            }
        )
        self._auth_header = {
            'Authorization': 'Bearer ' + response.json()['access_token']
        }
        self._access_token = response.json()['access_token']
        self._refresh_token = response.json()['refresh_token']
        return self._auth_header

    @auth_check
    def get_me(self) -> api_entities.User:
        response = requests.get(
            urljoin(self.env_vars.api_url, '/users/me'),
            headers=self._auth_header,
        )
        response.raise_for_status()
        resp = from_dict(data_class=api_entities.User, data=response.json())
        return resp

    @auth_check
    def create_mod(self, mod: api_entities.ModCreate) -> api_entities.ModBase:
        response = requests.post(
            urljoin(self.env_vars.api_url, '/mods/'),
            json=asdict(mod),
            headers=self._auth_header,
        )
        response.raise_for_status()

        res = from_dict(data_class=api_entities.ModBase, data=response.json())
        return res

    @auth_check
    def initiate_upload(
            self, mod_id: int,
            mod_version: api_entities.ModFileUploadRequest
    ) -> api_entities.ModFileUploadResponse:
        response = requests.post(
            urljoin(self.env_vars.api_url, f"/mods/{mod_id}/initiate_upload/"),
            json=asdict(mod_version),
            headers=self._auth_header,
        )
        response.raise_for_status()

        res = from_dict(data_class=api_entities.ModFileUploadResponse, data=response.json())
        return res

    @auth_check
    def my_mods(self):
        response = requests.get(
            urljoin(self.env_vars.api_url, "/mods/my/"),
            # self._api_url + f"/mods/my/",
            headers=self._auth_header,
        )
        response.raise_for_status()
        return response

    def upload_mod(self, mod_size: int, mod_file: bytes, policy_document: dict):
        object_id = policy_document['fields']['key']
        if self.env == APIEnv.local:
            response = self.local_upload(mod_file,
                                         object_name=object_id)

            md5_hash.update(mod_file)
            mod_md5_hash = base64.encodebytes(md5_hash.digest()).decode()
            notify_dedup_service(
                url=self.env_vars.modprocessing_url,
                upload_id=object_id,
                upload_size=mod_size,
                file_md5=mod_md5_hash,
                bucket_name=self.env_vars.upload_bucket_name
            )
        else:
            response = self.xml_upload(mod_file, policy_document, file_name=object_id)
        response.raise_for_status()

        return response



import os
import pathlib
from dataclasses import asdict
from pprint import pprint

import appdirs
import click
from requests import HTTPError

from . import __version__, Auth
from .api import APIEnv
from .api.api_entities import GamingPlatform
from .api.consumer import ConsumerClient, ConsumerClientException

def_data_dir = pathlib.Path(appdirs.user_data_dir(
    appname='vankrupt_workshop_client',
    appauthor="vankrupt",
    version=__version__
))
def_config_dir = pathlib.Path(appdirs.user_config_dir(
    appname='vankrupt_workshop_client',
    appauthor="vankrupt",
    version=__version__
))
def_conf_path = pathlib.Path.joinpath(def_config_dir, 'store.')


@click.group()
@click.version_option(version=__version__)
@click.option(
    '--env', '-e',
    type=click.Choice([APIEnv.dev.value, APIEnv.release.value, APIEnv.local.value], case_sensitive=True),
    default=APIEnv.dev.value
)
@click.pass_context
def cli(ctx, env):
    """Vankrupt workshop uploader client"""
    enum_env = APIEnv(env)
    ctx.ensure_object(dict)

    conf_file = str(def_conf_path) + enum_env.value
    if not pathlib.Path(conf_file).is_file():
        ...
        # print('First, you need to run "init" against this environment')
    else:
        with open(conf_file, 'r') as f:
            mod_home_dir = pathlib.Path(f.read())
        ctx.obj['mod_home_dir'] = mod_home_dir

    client = ConsumerClient(env=enum_env)
    ctx.obj['client'] = client
    ctx.obj['conf_file'] = conf_file


@cli.command()
@click.option(
    '--mod_home_dir', '-d',
    type=click.Path(exists=True),
    required=False, default=None
)
@click.option(
    '--env', '-e',
    type=click.Choice([APIEnv.dev.value, APIEnv.release.value, APIEnv.local.value], case_sensitive=True),
    default=APIEnv.dev.value
)
def init(env, mod_home_dir):
    if mod_home_dir:
        mod_home_dir = pathlib.Path(mod_home_dir)
    else:
        mod_home_dir = def_data_dir

    conf_file = str(def_conf_path) + env
    print(f'Writing {mod_home_dir} to conf file: {conf_file}')
    with open(conf_file, 'w+') as f:
        f.write(str(mod_home_dir))


@cli.command()
@click.option(
    '--env', '-e',
    type=click.Choice([APIEnv.dev.value, APIEnv.release.value, APIEnv.local.value], case_sensitive=True),
    default=APIEnv.dev.value
)
def deinit(env):
    conf_file = str(def_conf_path) + env
    pathlib.Path(conf_file).unlink(missing_ok=True)


@cli.command()
@click.option(
    '--mod_id', '-mi',
    type=int,
    required=True
)
@click.pass_context
def inspect_mod(ctx, mod_id):
    client: ConsumerClient
    client = ctx.obj['client']
    try:
        mod = client.inspect_mod(mod_id=mod_id)
    except HTTPError as e:
        print("Error:")
        pprint(e.response.json())
    else:
        pprint(asdict(mod))


@cli.command()
@click.option(
    '--page', '-p',
    type=int,
    required=True
)
@click.option(
    '--platform', '-pl',
    required=True,
    type=click.Choice(
        [
            GamingPlatform.linux.value, GamingPlatform.windows.value,
            GamingPlatform.ps5.value, GamingPlatform.android
        ],
        case_sensitive=True
    ),
)
@click.pass_context
def filter_mods(ctx, page, platform):
    size = 5
    platform = GamingPlatform(platform)

    client: ConsumerClient
    client = ctx.obj['client']
    try:
        mod = client.filter_mods(page=page, platform=platform, size=size)
    except HTTPError as e:
        print("Error:")
        pprint(e.response.json())
    else:
        pprint(asdict(mod))


def init_file_related(ctx, platform) -> (ConsumerClient, GamingPlatform, str):
    if 'mod_home_dir' not in ctx.obj.keys():
        print('Please run "init" command first.')
        exit()
    else:
        mod_home_dir = ctx.obj['mod_home_dir']
        platform = GamingPlatform(platform) if platform else None
        client: ConsumerClient
        client = ctx.obj['client']
        return client, platform, mod_home_dir


@cli.command()
@click.option(
    '--mod_id', '-mi',
    type=int,
    required=True
)
@click.option(
    '--platform', '-pl',
    required=True,
    type=click.Choice(
        [
            GamingPlatform.linux.value, GamingPlatform.windows.value,
            GamingPlatform.ps5.value, GamingPlatform.android
        ],
        case_sensitive=True
    ),
)
@click.pass_context
def download(ctx, mod_id, platform):
    client, platform, mod_home_dir = init_file_related(ctx, platform=platform)
    try:
        print(f'Will be saving the mod to {mod_home_dir}')
        client.save_mod(mods_home_dir=mod_home_dir, platform=platform, mod_id=mod_id)
    except IsADirectoryError as e:
        print(e)
    except HTTPError as e:
        print("Error:")
        pprint(e.response.json())
    else:
        print("Done!")


@cli.command()
@click.option(
    '--mod_id', '-mi',
    type=int,
    required=True
)
@click.option(
    '--platform', '-pl',
    required=True,
    type=click.Choice(
        [
            GamingPlatform.linux.value, GamingPlatform.windows.value,
            GamingPlatform.ps5.value, GamingPlatform.android
        ],
        case_sensitive=True
    ),
)
@click.pass_context
def update(ctx, mod_id, platform):
    client, platform, mod_home_dir = init_file_related(ctx, platform=platform)
    try:
        print(f'Will update the mod from {mod_home_dir}')
        client.update_mod(mods_home_dir=mod_home_dir, platform=platform, mod_id=mod_id)
    except NotADirectoryError as e:
        print(e)
    except HTTPError as e:
        print("Error:")
        pprint(e.response.json())
    except (FileNotFoundError, ConsumerClientException) as e:
        print("Downloaded probably corrupted, delete and download again, might help")
        print(f"Original error: {e}")
    else:
        print("Done!")


@cli.command()
@click.option(
    '--mod_id', '-mi',
    type=int,
    required=True
)
@click.option(
    '--platform', '-pl',
    required=True,
    type=click.Choice(
        [
            GamingPlatform.linux.value, GamingPlatform.windows.value,
            GamingPlatform.ps5.value, GamingPlatform.android
        ],
        case_sensitive=True
    ),
)
@click.pass_context
def delete(ctx, mod_id, platform):
    client, platform, mod_home_dir = init_file_related(ctx, platform=platform)
    client.delete_mod(mods_home_dir=mod_home_dir, mod_id=mod_id, platform=platform)


@cli.command()
@click.pass_context
def list_downloaded(ctx):
    def get_immediate_subdirectories(a_dir):
        return [name for name in os.listdir(a_dir)
                if os.path.isdir(os.path.join(a_dir, name))]

    client, _, mod_home_dir = init_file_related(ctx, platform=None)
    dirs = get_immediate_subdirectories(mod_home_dir)
    print('Got folders of following mods: ')
    print(dirs)



import json
import os
import pathlib
from dataclasses import asdict
from pprint import pprint

import appdirs
import click
from dacite import from_dict
from requests import HTTPError

from . import __version__, Auth
from .api import APIEnv
from .api.api_entities import ModCreate, ModFileUploadRequest, GamingPlatform
from .api.creator import CreatorClient, AuthError

def_config_dir = appdirs.user_config_dir(
    appname='vankrupt_workshop_client',
    appauthor="vankrupt",
    version=__version__
)
def_auth_path = pathlib.Path.joinpath(pathlib.Path(def_config_dir), 'auth.')


def rewrite_auth(client: CreatorClient):
    root = def_auth_path.parent
    pathlib.Path.mkdir(root, parents=True, exist_ok=True)
    conf_file = str(def_auth_path) + client.env.value
    with open(conf_file, 'w+') as config_f:
        a = Auth(access_token=client._access_token, refresh_token=client._refresh_token, env=client.env.value)
        s = json.dumps(asdict(a))
        config_f.write(s)


def delete_auth(client: CreatorClient):
    conf_file = str(def_auth_path) + client.env.value
    pathlib.Path.unlink(pathlib.Path(conf_file), missing_ok=True)


@click.group()
@click.version_option(version=__version__)
@click.option(
    '--config_store', '-c',
    type=click.File('rb'),
    default=None
)
@click.option(
    '--env', '-e',
    type=click.Choice([APIEnv.dev.value, APIEnv.release.value, APIEnv.local.value], case_sensitive=True),
    default=APIEnv.dev.value
)
@click.pass_context
def cli(ctx, config_store, env):
    """Vankrupt workshop uploader client"""
    enum_env = APIEnv(env)
    # try to read auth configs
    client = CreatorClient(env=enum_env)
    try:
        conf_file = str(def_auth_path) + enum_env.value
        if not config_store:
            config_store = open(conf_file, 'r')

        c_s = config_store.read()
        c_d = json.loads(c_s)
        auth_l = from_dict(data_class=Auth, data=c_d)
    except Exception as e:
        print(e)
    else:
        client._authenticate(a_t=auth_l.access_token, r_t=auth_l.refresh_token)

    ctx.ensure_object(dict)
    ctx.obj['client'] = client


@cli.command()
@click.option(
    '--string', '-s'
)
def echo(string):
    print(string)


@cli.command()
@click.option(
    '--email', '-e',
    prompt=True,
)
@click.option(
    "--password", '-p',
    prompt=True,
    hide_input=True,
)
@click.pass_context
def login(ctx, email, password):
    client: CreatorClient
    client = ctx.obj['client']
    try:
        client.authenticate(username=email, password=password)
    except Exception as e:
        print("Authentication failed")
        return
    print(f'Saving configs to root {def_auth_path.parent}')
    rewrite_auth(client=client)


@cli.command()
def logout(ctx):
    client: CreatorClient
    client = ctx.obj['client']
    delete_auth(client)


@cli.command()
@click.pass_context
def get_me(ctx):
    client: CreatorClient
    client = ctx.obj['client']
    if not client.authenticated:
        print("Unauthenticated, authenticate first.")
        return
    try:
        me = client.get_me()
        pprint(asdict(me), sort_dicts=False)
    except Exception as e:
        print('Got error:')
        raise e
    else:
        rewrite_auth(client)


@cli.command()
@click.option(
    '--name', '-n',
    required=True
)
@click.option(
    '--description', '-d',
    required=True
)
@click.pass_context
def create_mod(ctx, name, description):
    client: CreatorClient
    client = ctx.obj['client']
    if not client.authenticated:
        print("Unauthenticated, authenticate first.")
        return
    try:
        mod = client.create_mod(mod=ModCreate(name=name, description=description))
        pprint(asdict(mod), sort_dicts=False)
    except HTTPError as e:
        print('Got error:')
        pprint(e.response.json())
    except AuthError:
        print('Got auth error, logging out')
        delete_auth(client)
    else:
        rewrite_auth(client)


@cli.command()
@click.option(
    '--mod_id', '-mi',
    required=True
)
@click.option(
    '--platform', '-p',
    required=True,
    type=click.Choice(
        [
            GamingPlatform.linux.value, GamingPlatform.windows.value,
            GamingPlatform.ps5.value, GamingPlatform.android
        ],
        case_sensitive=True
    ),
)
@click.option(
    '--update_message', '-um',
    required=True
)
@click.option(
    '--file_path', '-f',
    type=click.Path(exists=True),
    required=True
)
@click.option(
    '--size', '-s',
    type=int,
    default=None
)
@click.pass_context
def upload(ctx, mod_id, platform, update_message, file_path, size):
    if not pathlib.Path(file_path).is_file():
        print("Not a file!")
        return
    real_size = os.path.getsize(file_path)
    if not size:
        size = real_size
    elif size > real_size:
        print(f"{file_path} is too small, it does not have {size} bytes")
        return

    client: CreatorClient
    client = ctx.obj['client']
    if not client.authenticated:
        print("Unauthenticated, authenticate first.")
        return

    try:
        new_v = ModFileUploadRequest(platform=GamingPlatform(platform), update_message=update_message, size=size)
        upl = client.initiate_upload(mod_id=mod_id, mod_version=new_v)
        pprint(upl)
        print("\n\n")
        print("Beginning uploading!")
        with open(file_path, 'rb') as f:
            bytes_to_send = f.read(size)

        resp = client.upload_mod(mod_size=size, mod_file=bytes_to_send, policy_document=upl.signed_policy)
        if resp.status_code == 204:
            print("Success!")

    except HTTPError as e:
        print('Got error:')
        pprint(e.response.json())
    except AuthError:
        print('Got auth error, logging out')
        delete_auth(client)
    else:
        rewrite_auth(client)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['vankrupt_workshop_client', 'vankrupt_workshop_client.api']

package_data = \
{'': ['*']}

install_requires = \
['appdirs>=1.4.4,<2.0.0',
 'click>=8.0.1,<9.0.0',
 'dacite>=1.6.0,<2.0.0',
 'requests>=2.26.0,<3.0.0']

entry_points = \
{'console_scripts': ['vankrupt_download = '
                     'vankrupt_workshop_client.consumer_cli:cli',
                     'vankrupt_upload = '
                     'vankrupt_workshop_client.creator_cli:cli']}

setup_kwargs = {
    'name': 'vankrupt-workshop-client',
    'version': '0.2.0',
    'description': 'Reference clients for vankrupt workshop',
    'long_description': "# Vankrupt workshop clients\nHere you'll find reference implementations of clients \nfor interacting with vankrupt workshop APIs.\n\nThe library comes with a relatively thin API and CLI clients for \nboth uploading mods, and downloading/upgrading them. \n\n## API library\nAPI client/library [readme](vankrupt_workshop_client/api/README.md) \nwill be most helpful for you, if you are looking for references \non how to implement uploader/downloader clients.\nThe documentation on how exactly to process downloads, updates, \nperform uploads is there as well.\n\n## CLI client\nCLI client [readme](vankrupt_workshop_client/README.md) \nis a good reference on how exactly to use API clients and tie them up\nto a concrete environment.\n\n\n## General info / status\nIt works against the environment you choose, both cli clients can be configured\nto work with `dev / prod / local` environments.\n\nCurrently it works against dev by default. When prod gets pulished, will \nwork with that.\n\n\n## Installation\nThe library is published to PyPi under the name \n[vankrupt-workshop-client](https://pypi.org/project/vankrupt-workshop-client).\n\nThus, to use it locally all you'd need to do is install it.\nI'd recommend to do it the following way:\n1. Create python virtual env:\n    ```shell script\n    python3 -m venv venv\n    ```\n2. Activate it:\n    ```shell script\n    source venv/bin/activate\n    ```\n3. Install:\n    ```shell script\n    pip install vankrupt-workshop-client\n    ```\n4. Use however you'd like, i.e.:\n    ```shell script\n    vankrupt_download filter-mods --page 1 --platform ps5\n    ```\n   For usage details, refer to \n   CLI client [readme](vankrupt_workshop_client/README.md).\n\nQuickstart: run\n1.\n    ```shell script\n    vankrupt_download --help\n    ```\n2.\n    ```shell script\n    vankrupt_upload --help\n    ```\n",
    'author': 'MeRuslan',
    'author_email': 'clanghopper@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import enum
import hashlib
import lzma
from dataclasses import dataclass
from string import Template
from typing import Optional, Callable


class APIEnv(enum.Enum):
    local = 'local'
    dev = 'dev'
    release = 'release'


@dataclass
class Env:
    api_url: str
    chunk_url_template: Template
    chunk_bucket_name: str = 'cdn.dev.vankrupt.io'
    upload_bucket_name: Optional[str] = None
    modprocessing_url: Optional[str] = None

    decompress_func: Callable = lzma.decompress
    hash_func: Callable = hashlib.sha3_256


_env_map = {
    APIEnv.local: Env(
        api_url='localhost:8000',
        chunk_url_template=Template('http://localhost:8081/storage/v1/b/cdn.dev.vankrupt.io/o/$mod_id/$hash'),
        modprocessing_url='http://localhost:9000/uploaded',
        upload_bucket_name='upload.dev.vankrupt.io',
    ),
    APIEnv.dev: Env(
        api_url='https://api.dev.vankrupt.io/',
        chunk_url_template=Template('https://storage.googleapis.com/cdn.dev.vankrupt.io/$mod_id/$hash')
    ),
    APIEnv.release: Env(
        api_url='https://api.vankrupt.io/',
        chunk_url_template=Template('https://storage.googleapis.com/cdn.dev.vankrupt.io/$mod_id/$hash')
    ),
}
